function Y = Generate_Input_feature_learning_3D_withMask(foldername, winsize,colorimage,idsformat,npatches,imageformat,downsample,rotation_angle)
%Function to generate the input samples for RBM..
%Input: foldername - folder containing images
%       winsize - length of image patch/block (assumed to be a square block)
%       colorimage - true for colorimages, false for grayscale
%       idsformat - true if images are in idsformat
%       npatches - number of patches to be extracted from each image
%       (random)
%       downsample - Rescaling the input image (optional)
%       rotation_angle - Angle by which input images have to be rotated
%       before extracting patches.
%Output: Y - A mxn matrix of input data to train the RBM. Here, m is the
%dimension of each input and n is the number of samples.

%Name of the directory to access image files
filenames = dir([foldername '/*.' imageformat]);
if not(exist('winsize','var'))
    w = 10;
else
    w = winsize;
end

if ~exist('colorimage','var')
    colorimage = false;
end

if ~exist('downsample','var')
    downsample = 1;
end
if colorimage
n = w*w*w*3; %Length of input vector
else
    n = w*w*w;
end

%Data matrix
Y = [];

for i = 1:length(filenames)
    
    %Read image
    if idsformat
        I = readIDS([foldername '/' filenames(i).name]);
    elseif strcmp(imageformat,'txt')
        I = readImageTXT([foldername '/' filenames(i).name]);
    else
        I = imread([foldername '/' filenames(i).name]);
    end
    [a1, a2, a3] = size(I);
    
    I = rescale(double(I),0,1); % Change to rescale per channel rather than all channel

    I = double(I);
    if exist('downsample','var')
        I = imresize(I,downsample);
    end
    if exist('rotation_angle','var')
        I = imrotate(I,rotation_angle);
    end
    disp(['Sampling random patches from: ' filenames(i).name]);
    [Y1 N1 N2 N3] = GetRandomPatches_3D_withMask(I,winsize,npatches);
    
    Y = [Y Y1];
end

return
