clear all; close all; clc

addpath spams-matlab
addpath spams-matlab/build
addpath minFunc

%% training dataset
datafolder = './ExampleData_PretrainedModel_ImagingBiomarkers/';
training_dataset = 'DB-Training-0330';
training_dataset_shortname = 'T0330';
training_datasetname = [datafolder '/' training_dataset '/'];

training_subtypename = {};
training_subdirs = dir(training_datasetname);
training_isub = [training_subdirs(:).isdir];
training_subtypename = {training_subdirs(training_isub).name}';
training_subtypename(ismember(training_subtypename,{'.','..'})) = [];

fl_top = length(training_subtypename);
foldernames_training = {};
for i = 1:fl_top
    foldernames_training{i} = [training_datasetname '/' training_subtypename{i} '/'];
end

%% biomarker extraction dataset
dataset = 'DB-BiomakerExtraction';
datasetname = [datafolder,'/' dataset '/'];

outputroot = datafolder;

subtypename = {};
subdirs = dir(datasetname);
isub = [subdirs(:).isdir];
subtypename = {subdirs(isub).name}';
subtypename(ismember(subtypename,{'.','..'})) = [];

foldernames = {};
for i = 1:length(subtypename)
    foldernames{i} = [datasetname '/' subtypename{i} '/'];
end

%% parameter settings
imageformat{1} = 'txt';

winsize = 20; % isotropic size 20 x 20 x 20 pixels
colorimage = false;
npatches = 100;
downsample = 1;
numlayers = 1;
nbasis_vec = [256];
lambda_vec = [30];
nepoch_vec = [100];
optmethod = 1; %<---- minfun = 1
whitenning = 0;
initialization = 1;
stepsize_for_feature_extraction = 5;


maxthresh=-1; % relative max per image (max(I(:))
minthresh=0.1; % relative min er image (0.1 * max(I(:)))

%% whether to use pre-trained imaging biomarkers or to re-train from example data
% 1) the following setting points to the pre-trained imaging biomarkers with exp_id = '_20200331_034504; 
% in order to use the pre-trained imaging biomarkers, the parameters settings above have to remain unchanged, since they were encoded in the model filename
% or,
% 2) comment off this setting to train your own imaging biomarkers if pre-trained imaging bomarkers are used for biomarker extraction, 

exp_id = '_20200331_034504'; 

%% setup experiments

outputfolder = [outputroot '/' training_dataset_shortname '_SPSD' '_step=' mat2str(stepsize_for_feature_extraction) '_maxt=' mat2str(maxthresh) '_mint=' mat2str(minthresh) '_nb=' mat2str(nbasis_vec) '_s=' mat2str(lambda_vec) '_ite=' mat2str(nepoch_vec) '_w' num2str(winsize) '_np=' num2str(npatches) '_d=' num2str(downsample) '_nL='];

modelfile = [outputroot '/' training_dataset_shortname '_Stacked_PSD'  '_nb=' mat2str(nbasis_vec) '_s=' mat2str(lambda_vec) '_ite=' mat2str(nepoch_vec) '_w' num2str(winsize) '_np=' num2str(npatches) '_d=' num2str(downsample) '_nL='];

disp(['Output folder prefix: ' outputfolder '\n']);
disp(['Model file prefix: ' modelfile '\n']);

%% imaging biomarker learning
if( ~exist('exp_id','var') ||  isempty(exp_id) )
    exp_id = Imaging_Biomarker_learning_StackPSD_3D(foldernames_training,imageformat,winsize,colorimage,npatches,modelfile,downsample,numlayers,nbasis_vec,lambda_vec,nepoch_vec,optmethod,whitenning,initialization)
end

%% imaging biomarker extraction
Imaging_Biomarker_extraction_StackPSD_3D(foldernames,outputfolder,modelfile,imageformat,winsize,colorimage,downsample,datasetname,subtypename,exp_id,whitenning,numlayers,stepsize_for_feature_extraction,maxthresh,minthresh);

disp('Imaging Biomarkers extracted successfully!')